@@ -3,10 +3,10 @@
  * SPDX-License-Identifier: Apache-2.0.
  */
 
-use aws_types::credentials;
 use std::borrow::Cow;
+use std::time::Duration;
 
-use aws_types::credentials::{future, ProvideCredentials};
+use aws_types::credentials::{self, future, ProvideCredentials};
 use tracing::Instrument;
 
 use crate::environment::credentials::EnvironmentVariableCredentialsProvider;
@@ -114,6 +114,71 @@ impl Builder {
         self
     }
 
+    /// Timeout for the entire credential loading chain.
+    ///
+    /// Defaults to 5 seconds.
+    pub fn load_timeout(mut self, timeout: Duration) -> Self {
+        self.set_load_timeout(Some(timeout));
+        self
+    }
+
+    /// Timeout for the entire credential loading chain.
+    ///
+    /// Defaults to 5 seconds.
+    pub fn set_load_timeout(&mut self, timeout: Option<Duration>) -> &mut Self {
+        self.credential_cache.set_load_timeout(timeout);
+        self
+    }
+
+    /// Amount of time before the actual credential expiration time
+    /// where credentials are considered expired.
+    ///
+    /// For example, if credentials are expiring in 15 minutes, and the buffer time is 10 seconds,
+    /// then any requests made after 14 minutes and 50 seconds will load new credentials.
+    ///
+    /// Defaults to 10 seconds.
+    pub fn buffer_time(mut self, buffer_time: Duration) -> Self {
+        self.set_buffer_time(Some(buffer_time));
+        self
+    }
+
+    /// Amount of time before the actual credential expiration time
+    /// where credentials are considered expired.
+    ///
+    /// For example, if credentials are expiring in 15 minutes, and the buffer time is 10 seconds,
+    /// then any requests made after 14 minutes and 50 seconds will load new credentials.
+    ///
+    /// Defaults to 10 seconds.
+    pub fn set_buffer_time(&mut self, buffer_time: Option<Duration>) -> &mut Self {
+        self.credential_cache.set_buffer_time(buffer_time);
+        self
+    }
+
+    /// Default expiration time to set on credentials if they don't have an expiration time.
+    ///
+    /// This is only used if the given [`ProvideCredentials`] returns
+    /// [`Credentials`](aws_types::Credentials) that don't have their `expiry` set.
+    /// This must be at least 15 minutes.
+    ///
+    /// Defaults to 15 minutes.
+    pub fn default_credential_expiration(mut self, duration: Duration) -> Self {
+        self.set_default_credential_expiration(Some(duration));
+        self
+    }
+
+    /// Default expiration time to set on credentials if they don't have an expiration time.
+    ///
+    /// This is only used if the given [`ProvideCredentials`] returns
+    /// [`Credentials`](aws_types::Credentials) that don't have their `expiry` set.
+    /// This must be at least 15 minutes.
+    ///
+    /// Defaults to 15 minutes.
+    pub fn set_default_credential_expiration(&mut self, duration: Option<Duration>) -> &mut Self {
+        self.credential_cache
+            .set_default_credential_expiration(duration);
+        self
+    }
+
     /// Add an additional credential source for the ProfileProvider
     ///
     /// Assume role profiles may specify named credential sources:
@@ -255,6 +320,7 @@ mod test {
     make_test!(web_identity_token_profile);
     make_test!(profile_name);
     make_test!(profile_overrides_web_identity);
+    make_test!(environment_variables_blank);
     make_test!(imds_token_fail);
 
     make_test!(imds_no_iam_role);
